from .base_config import BaseConfiguration
from .chatgpt_config import GPTConfiguration
from .dalle_config import DALLEConfiguration
from .hugging_face_config import HuggingFaceConfiguration
from .config_types import ConfigTypes
import os
import json


CONFIG_TYPE_TO_OBJECT = {
    ConfigTypes.CHATGPT: GPTConfiguration,
    ConfigTypes.DALLE: DALLEConfiguration,
    ConfigTypes.HUGGINGFACE: HuggingFaceConfiguration
}


def parse_models_configurations_from_file(config_file_path: str) -> dict[ConfigTypes, BaseConfiguration]:
    """
    Assumes the following JSON structure:
    {
        MODEL_NAME: {
            CONFIG_ATTRIBUTE: CONFIG_VALUE,
            ...
        }
    }

    For example
    {
        chatgpt: {
            api_key: ...,
            model: ...
        },
        dalle: {
            api_key: ...,
            num_of_images: ...,
            resolution: ...
        }
    }
    """
    if not config_file_path or not os.path.exists(config_file_path):
        print(f"Config file not set or doesn't exist, got: {config_file_path}, defaulting to runtime configurations")
        return []

    with open(config_file_path, "r") as f:
        config_json = json.load(f)

    
    models_configs: dict[ConfigTypes, BaseConfiguration] = {}

    for model_name in config_json.keys():
        config_object = CONFIG_TYPE_TO_OBJECT.get(model_name.upper())

        if not config_object:
            print(f"Unsupported model type {model_name}, skipping loading its configuration")
            continue

        try:
            config = config_object(**config_json[model_name])
            models_configs[model_name] = config
        except TypeError:
            print(f"Config of {model_name} is missing required fields, skipping loading its configuration")

    return models_configs



from typing import TypedDict, Dict, Any
import json
import requests
from .base_api_wrapper import RestAPIWrapper
from ..models_configurations.dalle_config import DALLEConfiguration

IMAGE_GENARATION_API_URL = "https://api.openai.com/v1/images/generations"


class ImageGenerationRequestBody(TypedDict):
    prompt: str
    n: int
    size: str


class DALLEWrapper(RestAPIWrapper):
    def __init__(self, prompt: str, configuration: DALLEConfiguration) -> None:
        super().__init__(prompt, configuration)
        self.api_url = IMAGE_GENARATION_API_URL
    
    def _get_body(self) -> ImageGenerationRequestBody:
        return json.dumps({
            "prompt": self.prompt,
            "n": self.config.num_of_images,
            "size": self.config.resolution
        })

    def _get_api_url(self) -> str:
        return self.api_url

    def _parse_response(self, response: Dict[str, Any]) -> str:
        images = []
        images_path = []
        data = response['data']
    
        for idx, item in enumerate(data):
            response = requests.get(item['url'])
            images.append(response.content)
            
            with open(f"image_{idx}.jpg", "wb") as f:
                f.write(response.content)
            images_path.append(f"image_{idx}.jpg")

        return {'images': images,
                'images_path': images_path}


from typing import TypedDict, Dict, Any
import json
from .base_api_wrapper import RestAPIWrapper
from ..models_configurations.hugging_face_config import HuggingFaceConfiguration

COMPLETIONS_API_URL = "https://api-inference.huggingface.co/models"

class GPTWrapperException(Exception):
    pass

class CompletionsRequestBody(TypedDict):
    prompt: str
    n: int
    size: str


class HuggingFaceWrapper(RestAPIWrapper):
    def __init__(self, prompt: str, configuration: HuggingFaceConfiguration) -> None:
        super().__init__(prompt, configuration)
        self.api_url = COMPLETIONS_API_URL
    
    def _get_body(self) -> CompletionsRequestBody:
        return json.dumps({
            "inputs": self.prompt,
        })

    def _get_api_url(self) -> str:
        return self.api_url + '/' + self.config.model_id

    def _parse_response(self, response: Dict[str, Any]) -> Dict[str, Any]:
        print (response)
        return response


from typing import Literal, TypedDict, Dict
import requests
import json

from ..models_configurations.chatgpt_config import GPTConfiguration, chatGptModel


COMPLETIONS_API_URL = "https://api.openai.com/v1/chat/completions"

class GPTWrapperException(Exception):
    pass

class CompletionsRequestBody(TypedDict):
    model: chatGptModel
    messages: list[dict[Literal["role", "content"], str]]


class GPTWrapper:
    def __init__(self, prompt: str, configuration: GPTConfiguration) -> None:
        self.prompt = prompt
        self.config = configuration

    def _get_headers(self) -> dict[str, str]:
        return {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.config.api_key}"
        }
    
    def _get_body(self) -> CompletionsRequestBody:
        return json.dumps({
            "model": self.config.model,
            "messages": [
                {"role": "user",
                 "content": self.prompt}
            ]
        })

    def __call__(self) -> Dict[str, str]:
        """
        raises GPTWrapperException if request failed
        returns string response from chatgpt completions api
        """
        response = requests.post(
            COMPLETIONS_API_URL,
            headers=self._get_headers(),
            data=self._get_body()
        )

        try:
            response.raise_for_status()
        except Exception as e:
            raise GPTWrapperException(f"Request to chatgpt completions api failed due to {e}. \n"
                                      f"Message response: {response.text}")
        
        return {'response': response.json()['choices'][0]['message']['content']}